from dataclasses import dataclass, field
from typing import Any, Optional

import pycountry
from moneyed import Money


class Address:
    def __init__(
        self: "Address",
        street: str,
        housenumber: str,
        zip_code: str,
        place: str,
        country: pycountry.db.Data,
        additional: Optional[str] = None,
        first_name: Optional[str] = None,
        last_name: Optional[str] = None,
        salutation: Optional[str] = None,
        title: Optional[str] = None,
        company: Optional[str] = None,
    ) -> None:
        self.street = street
        self.housenumber = housenumber
        self.zip_code = zip_code
        self.place = place
        self.country = country
        self.additional = additional
        self.first_name = first_name
        self.last_name = last_name
        self.salutation = salutation
        self.title = title
        self.company = company


class PersonAddress(Address):
    def __init__(
        self: "PersonAddress",
        first_name: str,
        last_name: str,
        *args: Any,
        **kwargs: Any,
    ) -> None:
        kwargs["first_name"] = first_name
        kwargs["last_name"] = last_name
        kwargs["company"] = None
        super().__init__(*args, **kwargs)


class CompanyAddress(Address):
    def __init__(
        self: "CompanyAddress", company: str, *args: Any, **kwargs: Any
    ) -> None:
        kwargs["company"] = company
        super().__init__(*args, **kwargs)


class StampProvider:
    name: str = ""

    def __init__(self: "StampProvider", *args: Any, **kwargs: Any) -> None:
        pass

    def get_products(self: "StampProvider") -> list["Product"]:
        raise NotImplementedError()

    def get_products_by_id(self: "StampProvider") -> dict[str, "Product"]:
        return {
            f"{self.name}_{product.product_id}": product
            for product in self.get_products()
        }

    def get_formats(self: "StampProvider") -> list["StampFormat"]:
        raise NotImplementedError()

    def get_formats_by_id(self: "StampProvider") -> dict[str, "StampFormat"]:
        return {
            f"{self.name}_{stamp_format.format_id}": stamp_format
            for stamp_format in self.get_formats()
        }

    def buy_stamp(
        self: "StampProvider",
        product: "Product",
        stamp_format: "StampFormat",
        sender: Address,
        receiver: Address,
    ) -> "Stamp":
        raise NotImplementedError()


@dataclass
class Stamp:
    """A single stamp."""

    product: "Product"
    stamp_format: "StampFormat"
    sender: Address
    receiver: Address
    pdf: bytes


@dataclass
class Product:
    """A stamp product."""

    provider: StampProvider
    product_id: str
    name: str
    price: Money
    max_weight: Optional[int] = None
    countries: Optional[list[pycountry.db.Data]] = field(
        default_factory=lambda: pycountry.countries
    )

    def __str__(self: "Product") -> str:
        return f"{self.name} ({self.price})"


@dataclass
class StampFormat:
    """A paper/label stamp_format."""

    provider: StampProvider
    format_id: str
    name: str
    page_type: str
    width: int
    height: int
    orientation: str
    description: str = ""
    labels_x: int = 1
    labels_y: int = 1
    with_address: bool = False
    with_image: bool = False

    def __str__(self: "StampFormat") -> str:
        return self.name



import pycountry
from inema import Internetmarke, inema
from moneyed import EUR, Money
from zeep.xsd.valueobjects import CompoundValue

from porto.provider import (
    Address,
    CompanyAddress,
    Product,
    Stamp,
    StampFormat,
    StampProvider,
)


class InternetmarkeProvider(StampProvider):
    name: str = "inema"

    def __init__(
        self: "InternetmarkeProvider",
        partner_id: str,
        key: str,
        key_phase: str,
        username: str,
        password: str,
    ) -> None:
        self.im = Internetmarke(partner_id, key, key_phase)
        self.im.authenticate(username, password)

    def get_products(self: "InternetmarkeProvider") -> list[Product]:
        """Get all products provided by Internetmarke."""
        products = []
        for product_id, product in self.im.products.items():
            price = Money(float(product["cost_price"]), EUR)
            max_weight = int(product["max_weight"]) or None
            if product["international"]:
                countries = pycountry.countries
            else:
                countries = [pycountry.countries.get(alpha_2="DE")]
            new_product = Product(
                provider=self,
                product_id=product_id,
                name=product["name"],
                price=price,
                max_weight=max_weight,
                countries=countries,
            )
            products.append(new_product)

        return products

    def get_formats(self: "InternetmarkeProvider") -> list["StampFormat"]:
        """Get all formats provided by Internetmarke."""
        formats = []
        for stamp_format in inema.formats:
            new_format = StampFormat(
                provider=self,
                format_id=str(stamp_format["id"]),
                name=stamp_format["name"],
                page_type=stamp_format["pageType"],
                width=stamp_format["pageLayout"]["size"]["x"],
                height=stamp_format["pageLayout"]["size"]["y"],
                orientation=stamp_format["pageLayout"]["orientation"],
                description=stamp_format["description"] or "",
                labels_x=stamp_format["pageLayout"]["labelCount"]["labelX"],
                labels_y=stamp_format["pageLayout"]["labelCount"]["labelY"],
                with_address=stamp_format["isAddressPossible"],
                with_image=stamp_format["isImagePossible"],
            )
            formats.append(new_format)
        return formats

    def _build_address(
        self: "InternetmarkeProvider", address: Address
    ) -> CompoundValue:
        """Convert address to a stamp_format compatible with Internetmarke API."""
        address_obj = self.im.build_addr(
            address.street,
            address.housenumber,
            address.zip_code,
            address.place,
            address.country.alpha_3,
            address.additional,
        )
        if isinstance(address, CompanyAddress):
            if address.first_name and address.last_name:
                pers_obj = self.im.build_pers_name(
                    address.first_name,
                    address.last_name,
                    address.salutation,
                    address.title,
                )
            else:
                pers_obj = None
            send_obj = self.im.build_comp_addr(address.company, address_obj, pers_obj)
        else:
            send_obj = self.im.build_pers_addr(
                address.first_name,
                address.last_name,
                address_obj,
                address.salutation,
                address.title,
            )
        return send_obj

    def buy_stamp(
        self: "InternetmarkeProvider",
        product: Product,
        stamp_format: StampFormat,
        sender: Address,
        receiver: Address,
    ) -> Stamp:
        """Buy and create a single stamp using Internetmarke."""
        sender_obj, receiver_obj = self._build_address(sender), self._build_address(
            receiver
        )
        pos = self.im.build_position(
            product.product_id, sender=sender_obj, receiver=receiver_obj, pdf=True
        )
        self.im.add_position(pos)
        stamp = self.im.checkoutPDF(stamp_format.format_id)
        return Stamp(
            pdf=stamp.pdf_bin,
            product=product,
            stamp_format=stamp_format,
            sender=sender,
            receiver=receiver,
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['porto', 'porto.providers']

package_data = \
{'': ['*']}

install_requires = \
['inema>=0.8.7,<0.9.0', 'py-moneyed>=2.0,<3.0', 'pycountry>=22.3.5,<23.0.0']

setup_kwargs = {
    'name': 'porto',
    'version': '0.1.2',
    'description': 'Porto provides a unified and simple interface to various online stamp services.',
    'long_description': 'Porto\n=====\n\nPorto provides an unified and simplified access to various online stamp services. Currently supported are the following services:\n\n* Deutsche Post: Internetmarke (Germany)\n\nInstallation\n------------\n\nThe simplest way to install Porto is using pip:\n\n..\n\n  pip install porto\n\nAdditionally, you can checkout the source code:\n\n..\n\n  git clone https://edugit.org/hansegucker/porto\n\nLicense\n-------\n\nCopyright 2022 Jonathan Weth <dev@jonathanweth.de>\n\nLicensed under the MIT License (see LICENSE for more information).\n',
    'author': 'Jonathan Weth',
    'author_email': 'dev@jonathanweth.de',
    'maintainer': 'Jonathan Weth',
    'maintainer_email': 'dev@jonathanweth.de',
    'url': 'https://edugit.org/hansegucker/porto',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
